# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.2.1

import enum

import winrt.system
from winrt._winrt_windows_devices_haptics import (
    KnownSimpleHapticsControllerWaveforms,
    SimpleHapticsController,
    SimpleHapticsControllerFeedback,
    VibrationDevice,
)

__all__ = [
    "VibrationAccessStatus",
    "KnownSimpleHapticsControllerWaveforms",
    "SimpleHapticsController",
    "SimpleHapticsControllerFeedback",
    "VibrationDevice",
]

class VibrationAccessStatus(enum.IntEnum):
    ALLOWED = 0
    DENIED_BY_USER = 1
    DENIED_BY_SYSTEM = 2
    DENIED_BY_ENERGY_SAVER = 3

